require 'test_helper'

describe Forward::API::Resource do
  before :each do
    Forward::Config.accounts = { foo: 'abc123' }
    Forward::Config.default_account = :foo
  end

  it 'builds http requests based on given method' do
    resource = Forward::API::Resource.new

    [ :get, :post, :delete ].each do |method|
      options = {}

      resource.expects(:request).with(method, options)
      resource.send(method, options)
    end
  end

  it "adds json headers to every request" do
    resource = Forward::API::Resource.new
    headers  = {
      'Accept'       => 'application/json',
      'Content-Type' => 'application/json'
    }

    stub_request(:get, 'localhost/').
      with(headers: headers).
      to_return(status: 200)

    EM.run_block {
      Forward::API::Resource.new.get(path: '/') {}
    }
  end

  it "adds auth header if request is authenticated" do
    resource = Forward::API::Resource.new
    headers  = {
      'Accept'        => 'application/json',
      'Authorization' => 'Token token=abc123',
      'Content-Type'  => 'application/json'
    }

    stub_request(:get, 'localhost/').
      with(headers: headers).
      to_return(status: 200)

    EM.run_block {
      Forward::API::Resource.new.get(path: '/', authenticated: true) {}
    }
  end

  it "adds a json body if params given" do
    resource = Forward::API::Resource.new
    params   = { foo: 'bar' }

    stub_request(:post, 'localhost/').
      with(body: params.to_json).
        to_return(status: 200)

    EM.run_block {
      Forward::API::Resource.new.post(path: '/', params: params) {}
    }
  end

  it "exits with message if status is 401" do
    resource = Forward::API::Resource.new

    stub_request(:get, 'localhost/').
      to_return(status: 401)

    out, err = capture_io do
      begin
        EM.run_block {
          Forward::API::Resource.new.get(path: '/') {}
        }
      rescue SystemExit; end
    end
    out.must_match /Unable to authenticate/i
  end

  it "exits with message if response is invalid JSON" do
    resource = Forward::API::Resource.new

    stub_request(:get, 'localhost/').
      to_return(status: 200, body: '{"foo"}')

    out, err = capture_io do
      begin
        EM.run_block {
          Forward::API::Resource.new.get(path: '/') {}
        }
      rescue SystemExit; end
    end
    out.must_match /Unable to connect to API/i
  end

end
