require 'test_helper'

describe Forward::Command::Tunnel do
  before :each do
    FakeFS.activate!
    FileUtils.mkdir_p(ENV['HOME'])
  end

  after :each do
    FileUtils.rm_rf(ENV['HOME'])
    FakeFS.deactivate!
  end

  it 'parses a forwarded port' do
    command = Forward::Command::Tunnel.new({}, ['600'])

    command.send(:parse_forwarded)
    command.options[:port].must_equal 600
  end

  it 'parses a forwarded host' do
    command = Forward::Command::Tunnel.new({}, ['mysite.dev'])

    command.send(:parse_forwarded)
    command.options[:host].must_equal 'mysite.dev'
  end

  it 'parses a forwarded host and port' do
    command = Forward::Command::Tunnel.new({}, ['mysite.dev:8888'])

    command.send(:parse_forwarded)
    command.options[:host].must_equal 'mysite.dev'
    command.options[:port].must_equal 8888
  end

  it 'exits if auth pair is invalid' do
    [ 'as:df:asdf', 'fooo bar', 'foo oo:bar' ].each do |auth|
      lambda {
        dev_null {
          command = Forward::Command::Tunnel.new(auth: auth)
          command.send(:validate_auth)
        }
      }.must_raise Forward::CLIError
    end
  end

  it 'validates a auth pair' do
    [ 'foo:asdflkj3r&)', 'tom:#SFSDF' ].each do |auth|
      command = Forward::Command::Tunnel.new(auth: auth)
      command.send(:validate_auth)
    end
  end

  it 'parses a Forwardfile' do
    hash_to_forwardfile(auth: 'username:password')

    command = Forward::Command::Tunnel.new
    command.send(:parse_forwardfile)
    command.options[:auth].must_equal 'username:password'
  end

  it 'exits if a Forwardfile does not parse to a Hash' do
    hash_to_forwardfile('username:password')

    lambda {
      dev_null { Forward::Command::Tunnel.new.send(:parse_forwardfile) }
    }.must_raise SystemExit
  end

  it 'overloads Forwardfile options via commandline options' do
    hash_to_forwardfile(
      port:             8000,
      username:         'username',
      password:         'password',
      subdomain_prefix: 'foo'
    )

    EM.expects(:run)

    command = Forward::Command::Tunnel.new({auth: 'example:secret'}, ['3000', 'bar'])
    command.start

    command.options[:port].must_equal 3000
    command.options[:username].must_equal 'example'
    command.options[:password].must_equal 'secret'
    command.options[:subdomain_prefix].must_equal 'bar'
  end

  it 'validates port and exits if invalid' do
    command         = Forward::Command::Tunnel.new
    command.options = { port: 3000 }

    command.send(:validate_port)

    [ 0, 65536 ].each do |port|
      lambda {
        dev_null {
          command.options[:port] = port
          command.send(:validate_port) }
      }.must_raise Forward::CLIError
    end
  end

  it 'validates cname and exits if invalid' do
    command         = Forward::Command::Tunnel.new
    command.options = { cname: 'foo.bar.com' }

    command.send(:validate_cname)
    [ 'whatever', 'asdfasdf.', '-asdf', 'adsf#$).com' ].each do |cname|
      lambda {
        dev_null {
          command.options[:cname] = cname
          command.send(:validate_cname)
        }
      }.must_raise Forward::CLIError
    end
  end

  it 'validates subdomain prefix and exits if invalid' do
    command         = Forward::Command::Tunnel.new
    command.options = { subdomain_prefix: 'foo' }

    command.send(:validate_subdomain_prefix)
    [ '-asdf', 'adsf#$)' ].each do |subdomain|
      lambda {
        dev_null {
          command.options[:subdomain_prefix] = subdomain
          command.send(:validate_subdomain_prefix)
        }
      }.must_raise Forward::CLIError
    end
  end
end
