require 'test_helper'

describe Forward::Config do

  before :each do
    @config = Forward::Config

    FakeFS.activate!
    FileUtils.mkdir_p(ENV['HOME'])
  end

  after :each do
    FileUtils.rm_rf(ENV['HOME'])
    FakeFS.deactivate!
  end

  it "sets default config values" do
    @config.set_defaults!
    @config::DEFAULTS.each do |setting, value|
      @config.send(setting).must_equal value
    end
  end

  it "sets a default config value" do
    @config.set_default!(:auto_open)
    @config.auto_open.must_equal @config::DEFAULTS[:auto_open]
  end

  it "has a windows config path if windows" do
    Forward.expects(:windows?).returns(true)
    @config.config_path.must_match /config\z/
  end

  it "has a non-windows config path if windows" do
    Forward.expects(:windows?).returns(false)
    @config.config_path.must_match /\.forwardrc\z/
  end
  #
  it "updates a config with a Hash" do
    @config.set_defaults!
    @config.update(auto_copy: false, auto_open: true)

    @config.auto_copy.must_equal false
    @config.auto_open.must_equal true
  end

  it "converts current config to a Hash" do
    @config.set_defaults!

    @config.to_hash.must_equal @config::DEFAULTS
  end

  it "knows if a config file is present" do
      @config.wont_be :present?
      FileUtils.touch(@config.config_path)
      @config.must_be :present?
  end

  it "writes config to config file" do
    @config.set_defaults!

    @config.wont_be :present?
    @config.write
    @config.must_be :present?
    YAML.load_file(@config.config_path).must_equal @config::DEFAULTS
  end

  it "loads a config file" do
    File.open(@config.config_path, 'w') { |f|
      f.write(YAML.dump(@config::DEFAULTS))
    }

    @config.must_be :present?
    @config.load
    @config::DEFAULTS.each do |setting, value|
      @config.send(setting).must_equal value
    end
  end

  it "adds an initial account" do
    @config.set_defaults!
    account = { foo: '1234' }
    subdomain, api_key = account.first

    @config.add_account(subdomain, api_key)
    @config.accounts.wont_be :empty?
    @config.accounts.has_key?(subdomain).must_equal true
    @config.default_account.must_equal subdomain
  end

  it "removes an account and sets the default account to the next available" do
    @config.set_defaults!
    accounts = { foo: '1234', bar: '4321' }

    accounts.each do |subdomain, api_key|
      @config.add_account(subdomain, api_key)
    end

    @config.default_account.must_equal accounts.keys.last
    @config.accounts.size.must_equal accounts.size

    @config.remove_account(accounts.keys.last)

    @config.accounts.size.must_equal accounts.size - 1
    @config.default_account.must_equal accounts.keys.first
  end

  it "removes the last account and sets default account to nil" do
    @config.set_defaults!
    account = { foo: '1234' }
    subdomain, api_key = account.first

    @config.add_account(subdomain, api_key)
    @config.remove_account(subdomain)

    @config.accounts.must_be :empty?
    @config.default_account.must_be_nil
  end

  it "sets the default account to an existing account" do
    @config.set_defaults!

    accounts        = { foo: '1234', bar: '4321' }
    default_account = accounts.keys.last

    accounts.each do |subdomain, api_key|
      @config.add_account(subdomain, api_key)
    end

    @config.accounts.size.must_equal accounts.size
    @config.set_default_account default_account
    @config.default_account.must_equal default_account
  end

  it "gets the api token from the default account" do
    @config.set_defaults!
    account = { foo: '1234' }
    subdomain, api_key = account.first

    @config.add_account(subdomain, api_key)
    @config.api_key.must_equal api_key
  end

end
